@@ -182,7 +182,6 @@ static void init(grib_accessor* a,const long v, grib_arguments* args)
     a->flags |= GRIB_ACCESSOR_FLAG_DATA;
 }
 
-
 static int value_count(grib_accessor* a,long* count)
 {
     grib_accessor_data_complex_packing *self =(grib_accessor_data_complex_packing*)a;
@@ -247,7 +246,6 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
     long   binary_scale_factor         = 0;
     long   decimal_scale_factor = 0;
 
-
     long   sub_j= 0;
     long   sub_k= 0;
     long   sub_m= 0;
@@ -407,6 +405,8 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
                 reference_value,s,pscals,(maxv-hcount)*2,pval);
         i+=(maxv-hcount)*2;
 #else
+        (void)pscals; /* suppress gcc warning */
+        (void)pval;   /* suppress gcc warning */
         for(lcount=hcount; lcount < maxv ; lcount++)
         {
             val[i++] =  (double) ((grib_decode_unsigned_long(lres, &lpos,
@@ -433,11 +433,9 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
     grib_context_free(a->parent->h->context,scals);
 
     return ret;
-
 }
 
 
-
 #define MAXVAL(a,b) a>b?a:b
 
 static double calculate_pfactor(grib_context *ctx,const double* spectralField, long fieldTruncation, long subsetTruncation)
@@ -564,11 +562,9 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len) {
     unsigned char* buf    = NULL;
 
     size_t         buflen = 0;
-
     size_t         hsize = 0;
     size_t         lsize = 0;
 
-
     unsigned char* hres = NULL;
     unsigned char* lres = NULL;
 
