@@ -40,7 +40,23 @@ static void init()
     pthread_mutex_init(&mutex_mem,&attr);
     pthread_mutexattr_destroy(&attr);
 }
+#elif GRIB_OMP_THREADS
+static int once = 0;
+static omp_nest_lock_t mutex_mem;
+static omp_nest_lock_t mutex_c;
 
+static void init()
+{
+    GRIB_OMP_CRITICAL(lock_grib_context_c)
+    {
+        if (once == 0)
+        {
+            omp_init_nest_lock(&mutex_mem);
+            omp_init_nest_lock(&mutex_c);
+            once = 1;
+        }
+    }
+}
 #endif
 
 
@@ -52,7 +68,7 @@ static long cntp = 0;
 
 static void default_long_lasting_free(const grib_context* c, void* p)
 {
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     free(p);
     GRIB_MUTEX_LOCK(&mutex_mem);
     cntp--;
@@ -62,7 +78,7 @@ static void default_long_lasting_free(const grib_context* c, void* p)
 static void* default_long_lasting_malloc(const grib_context* c, size_t size)
 {
     void* ret;
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     GRIB_MUTEX_LOCK(&mutex_mem);
     cntp++;
     GRIB_MUTEX_UNLOCK(&mutex_mem);
@@ -73,7 +89,7 @@ static void* default_long_lasting_malloc(const grib_context* c, size_t size)
 
 static void default_buffer_free(const grib_context* c, void* p)
 {
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     free(p);
     GRIB_MUTEX_LOCK(&mutex_mem);
     cntp--;
@@ -83,7 +99,7 @@ static void default_buffer_free(const grib_context* c, void* p)
 static void* default_buffer_malloc(const grib_context* c, size_t size)
 {
     void* ret;
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     GRIB_MUTEX_LOCK(&mutex_mem);
     cntp++;
     GRIB_MUTEX_UNLOCK(&mutex_mem);
@@ -102,7 +118,7 @@ static void* default_buffer_realloc(const grib_context* c, void* p, size_t size)
 
 static void default_free(const grib_context* c, void* p)
 {
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     free(p);
     GRIB_MUTEX_LOCK(&mutex_mem);
     cnt--;
@@ -112,7 +128,7 @@ static void default_free(const grib_context* c, void* p)
 static void* default_malloc(const grib_context* c, size_t size)
 {
     void* ret;
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     GRIB_MUTEX_LOCK(&mutex_mem);
     cnt++;
     GRIB_MUTEX_UNLOCK(&mutex_mem);
@@ -246,6 +262,11 @@ void grib_print_api_version(FILE* out)
     }
 }
 
+const char* grib_get_package_name(void)
+{
+    return "grib_api";
+}
+
 static grib_context default_grib_context = {
         0,                            /* inited                     */
         0,                            /* debug                      */
@@ -328,7 +349,8 @@ static grib_context default_grib_context = {
 
 grib_context* grib_context_get_default()
 {
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
+    GRIB_MUTEX_LOCK(&mutex_c);
 
 #ifdef ENABLE_FLOATING_POINT_EXCEPTIONS
     feenableexcept(FE_ALL_EXCEPT & ~FE_INEXACT);
@@ -348,8 +370,6 @@ grib_context* grib_context_get_default()
         const char *no_spd=NULL;
         const char *keep_matrix=NULL;
 
-        GRIB_MUTEX_LOCK(&mutex_c);
-
         write_on_fail = getenv("GRIB_API_WRITE_ON_FAIL");
         large_constant_fields = getenv("GRIB_API_LARGE_CONSTANT_FIELDS");
         no_abort = getenv("GRIB_API_NO_ABORT");
@@ -442,10 +462,9 @@ grib_context* grib_context_get_default()
                 &(default_grib_context.concepts_count));
         default_grib_context.def_files=grib_trie_new(&(default_grib_context));
         default_grib_context.classes=grib_trie_new(&(default_grib_context));
-
-        GRIB_MUTEX_UNLOCK(&mutex_c);
     }
 
+    GRIB_MUTEX_UNLOCK(&mutex_c);
     return &default_grib_context;
 }
 
@@ -460,7 +479,7 @@ grib_context* grib_context_new(grib_context* parent)
 
     if (!parent) parent=grib_context_get_default();
 
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     GRIB_MUTEX_LOCK(&(parent->mutex));
 
     c = (grib_context*)grib_context_malloc_clear_persistent(&default_grib_context,sizeof(grib_context));
@@ -537,7 +556,7 @@ static int init_definition_files_dir(grib_context* c)
     /* Note: strtok modifies its first argument so we copy */
     strncpy(path, c->grib_definition_files_path, DEF_PATH_MAXLEN);
 
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     GRIB_MUTEX_LOCK(&mutex_c);
 
     p=path;
@@ -579,7 +598,7 @@ char *grib_context_full_defs_path(grib_context* c,const char* basename)
     grib_string_list* fullpath=0;
     if (!c) c=grib_context_get_default();
 
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
 
     if(*basename == '/' || *basename ==  '.') {
         return (char*)basename;
@@ -871,3 +890,39 @@ void grib_context_print(const grib_context *c, void* descriptor,const char* fmt,
     va_end(list);
     c->print(c,descriptor,msg);
 }
+
+void grib_context_set_handle_file_count(grib_context *c, int new_count)
+{
+    if (!c) c=grib_context_get_default();
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
+    GRIB_MUTEX_LOCK(&mutex_c);
+    c->handle_file_count = new_count;
+    GRIB_MUTEX_UNLOCK(&mutex_c);
+}
+
+void grib_context_set_handle_total_count(grib_context *c, int new_count)
+{
+    if (!c) c=grib_context_get_default();
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
+    GRIB_MUTEX_LOCK(&mutex_c);
+    c->handle_total_count = new_count;
+    GRIB_MUTEX_UNLOCK(&mutex_c);
+}
+
+void grib_context_increment_handle_file_count(grib_context *c)
+{
+    if (!c) c=grib_context_get_default();
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
+    GRIB_MUTEX_LOCK(&mutex_c);
+    c->handle_file_count++;
+    GRIB_MUTEX_UNLOCK(&mutex_c);
+}
+
+void grib_context_increment_handle_total_count(grib_context *c)
+{
+    if (!c) c=grib_context_get_default();
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
+    GRIB_MUTEX_LOCK(&mutex_c);
+    c->handle_total_count++;
+    GRIB_MUTEX_UNLOCK(&mutex_c);
+}
