@@ -45,6 +45,29 @@ static void init()
     pthread_mutex_init(&mutex_parse,&attr);
     pthread_mutexattr_destroy(&attr);
 }
+#elif GRIB_OMP_THREADS
+static int once = 0;
+static omp_nest_lock_t mutex_file;
+static omp_nest_lock_t mutex_rules;
+static omp_nest_lock_t mutex_concept;
+static omp_nest_lock_t mutex_stream;
+static omp_nest_lock_t mutex_parse;
+
+static void init()
+{
+    GRIB_OMP_CRITICAL(lock_grib_parse_utils_c)
+    {
+        if (once == 0)
+        {
+            omp_init_nest_lock(&mutex_file);
+            omp_init_nest_lock(&mutex_rules);
+            omp_init_nest_lock(&mutex_concept);
+            omp_init_nest_lock(&mutex_stream);
+            omp_init_nest_lock(&mutex_parse);
+            once = 1;
+        }
+    }
+}
 #endif
 
 int grib_recompose_name(grib_handle* h, grib_accessor *observer, const char* uname, char* fname,int fail)
@@ -395,48 +418,45 @@ int grib_yyerror(const char* msg)
 {
     grib_context_log(grib_parser_context, GRIB_LOG_ERROR,
             "grib_parser: %s at line %d of %s", msg, grib_yylineno + 1,parse_file);
+    grib_context_log(grib_parser_context, GRIB_LOG_ERROR,
+            "GRIB API Version: %s", GRIB_API_VERSION_STR);
     error = 1;
     return 1;
 }
 
-void grib_parser_include(const char* fname)
+void grib_parser_include(const char* included_fname)
 {
     FILE *f = NULL;
-    char path[1204];
     char* io_buffer=0;
     /* int i; */
     Assert(top < MAXINCLUDE);
-    Assert(fname);
+    Assert(included_fname);
 
     if(parse_file == 0)
     {
-        parse_file = fname;
+        parse_file = included_fname;
         Assert(top == 0);
     }
     else
     {
-        const char *p = parse_file;
-        const char *q = NULL;
+        /* When parse_file is not NULL, it's the path of the parent file (includer) */
+        /* and 'included_fname' is the name of the file being included (includee) */
 
-        while(*p) {
-            if(*p == '/') q = p;
-            p++;
-        }
+        /* GRIB-796: Search for the included file in GRIB_DEFINITION_PATH */
+        char* new_path = NULL;
+        Assert(*included_fname != '/');
+        new_path = grib_context_full_defs_path(grib_parser_context, included_fname);
+        if (!new_path) {
+            fprintf(stderr, "GRIB API Version:      %s\nDefinition files path: %s\n",
+                    GRIB_API_VERSION_STR,
+                    grib_parser_context->grib_definition_files_path);
 
-        if(!q) {
             grib_context_log(grib_parser_context, GRIB_LOG_FATAL,
-                    "grib_parser_include: path '%s' does not contain a '/'\n",fname);
+                    "grib_parser_include: Could not resolve '%s' (included in %s)", included_fname, parse_file);
+
             return;
         }
-        q++;
-
-        strncpy(path,parse_file,q-parse_file);
-        path[q-parse_file] = 0;
-        strcat(path,fname);
-
-        Assert(*fname != '/');
-
-        parse_file = path;
+        parse_file = new_path;
     }
 
     if (strcmp(parse_file,"-")==0) {
@@ -467,14 +487,13 @@ void grib_parser_include(const char* fname)
             setvbuf(f,io_buffer,_IOFBF,c->io_buffer_size);
         }
         */
-
-        grib_yyin            = f;
+        grib_yyin       = f;
         stack[top].file = f;
         stack[top].io_buffer = io_buffer;
         stack[top].name = grib_context_strdup(grib_parser_context,parse_file);
         parse_file      = stack[top].name;
         stack[top].line = grib_yylineno;
-        grib_yylineno = 0;
+        grib_yylineno   = 0;
         top++;
         /* grib_yyrestart(f); */
     }
@@ -485,7 +504,7 @@ extern int grib_yyparse(void);
 static int parse(grib_context* gc, const char* filename)
 {
     int err = 0;
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     GRIB_MUTEX_LOCK(&mutex_parse);
 
 #ifdef YYDEBUG
@@ -518,7 +537,7 @@ static int parse(grib_context* gc, const char* filename)
 
 static grib_action* grib_parse_stream(grib_context* gc, const char* filename)
 {
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     GRIB_MUTEX_LOCK(&mutex_stream);
 
     grib_parser_all_actions = 0;
@@ -540,7 +559,7 @@ static grib_action* grib_parse_stream(grib_context* gc, const char* filename)
 
 grib_concept_value* grib_parse_concept_file( grib_context* gc,const char* filename)
 {
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     GRIB_MUTEX_LOCK(&mutex_concept);
 
     gc = gc ? gc : grib_context_get_default();
@@ -559,7 +578,7 @@ grib_rule* grib_parse_rules_file( grib_context* gc,const char* filename)
 {
     if (!gc) gc=grib_context_get_default();
 
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     GRIB_MUTEX_LOCK(&mutex_rules);
 
     gc = gc ? gc : grib_context_get_default();
@@ -578,7 +597,7 @@ grib_action* grib_parse_file( grib_context* gc,const char* filename)
 {
     grib_action_file* af;
 
-    GRIB_PTHREAD_ONCE(&once,&init);
+    GRIB_MUTEX_INIT_ONCE(&once,&init);
     GRIB_MUTEX_LOCK(&mutex_file);
 
     af =0;
