@@ -351,18 +351,32 @@ int grib_set_string_internal(grib_handle* h, const char* name,
 int grib_set_string(grib_handle* h, const char* name, const char* val, size_t *length)
 {
     int ret=0;
-    grib_accessor* a;
+    grib_accessor* a = NULL;
 
-#if 1
-    /*second order doesn't have a proper representation for constant fields
-  the best is not to do the change of packing type if bitsPerValue=0
-     */
-    if (!grib_inline_strcmp(name,"packingType") && !grib_inline_strcmp(val,"grid_second_order")) {
+    /* Second order doesn't have a proper representation for constant fields.
+       So best not to do the change of packing type.
+       Use strncmp to catch all flavours of second order packing e.g. grid_second_order_boustrophedonic
+    */
+    if (!grib_inline_strcmp(name,"packingType") && !strncmp(val,"grid_second_order", 17)) {
         long bitsPerValue=0;
+        size_t numCodedVals = 0;
         grib_get_long(h,"bitsPerValue",&bitsPerValue);
-        if (bitsPerValue==0) return 0;
+        if (bitsPerValue==0) {
+            if (h->context->debug) {
+                printf("GRIB_API DEBUG grib_set_string packingType: Constant field cannot be encoded in second order. Packing not changed\n");
+            }
+            return 0;
+        }
+
+        /* GRIB-883: check if there are enough coded values */
+        ret = grib_get_size(h, "codedValues", &numCodedVals);
+        if (ret == GRIB_SUCCESS && numCodedVals < 3) {
+            if (h->context->debug) {
+                printf("GRIB_API DEBUG grib_set_string packingType: not enough coded values for second order. Packing not changed\n");
+            }
+            return 0;
+        }
     }
-#endif
 
     a = grib_find_accessor(h, name);
 
@@ -642,15 +656,22 @@ static int __grib_set_double_array(grib_handle* h, const char* name, const doubl
 
             grib_get_string(h,"packingType",packingType,&slen);
             if (!strcmp(packingType,"grid_second_order") ||
-                    !strcmp(packingType,"grid_second_order_no_SPD") ||
-                    !strcmp(packingType,"grid_second_order_SPD1") ||
-                    !strcmp(packingType,"grid_second_order_SPD2") ||
-                    !strcmp(packingType,"grid_second_order_SPD3")
+                !strcmp(packingType,"grid_second_order_no_SPD") ||
+                !strcmp(packingType,"grid_second_order_SPD1") ||
+                !strcmp(packingType,"grid_second_order_SPD2") ||
+                !strcmp(packingType,"grid_second_order_SPD3")
             ) {
+                int ret = 0;
                 slen=11; /*length of 'grid_simple' */
-                if (h->context->debug)
-                    printf("GRIB_API DEBUG grib_set_double_array forcing grid_simple\n");
-                grib_set_string(h,"packingType","grid_simple",&slen);
+                if (h->context->debug) {
+                    printf("GRIB_API DEBUG __grib_set_double_array: Cannot use second order packing for constant fields. Using simple packing\n");
+                }
+                ret = grib_set_string(h,"packingType","grid_simple",&slen);
+                if (ret != GRIB_SUCCESS) {
+                    if (h->context->debug) {
+                        printf("GRIB_API DEBUG __grib_set_double_array: could not switch to simple packing!\n");
+                    }
+                }
             }
         }
     }
